/********************************************************************************

USI TWI Slave driver.

Created by Donald R. Blake
donblake at worldnet.att.net

---------------------------------------------------------------------------------

Created from Atmel source files for Application Note AVR312: Using the USI Module
as an I2C slave.

This program is free software; you can redistribute it and/or modify it under the
terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

---------------------------------------------------------------------------------

Change Activity:

    Date       Description
   ------      -------------
  16 Mar 2007  Created.
  27 Mar 2007  Added support for ATtiny261, 461 and 861.
  26 Apr 2007  Fixed ACK of slave address on a read.

********************************************************************************/



/********************************************************************************

                                    includes

********************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "usiTwiSlave.h"
#include "util/atomic.h"
#include <util/delay.h>

/********************************************************************************

                            device dependent defines

********************************************************************************/

#if defined( __AVR_ATtiny2313__ )
#  define DDR_USI             DDRB
#  define PORT_USI            PORTB
#  define PIN_USI             PINB
#  define PORT_USI_SDA        PB5
#  define PORT_USI_SCL        PB7
#  define PIN_USI_SDA         PINB5
#  define PIN_USI_SCL         PINB7
#  define USI_START_COND_INT  USISIF
#  define USI_START_VECTOR    USI_START_vect
#  define USI_OVERFLOW_VECTOR USI_OVERFLOW_vect
#endif

#if defined( __AVR_ATtiny25__ ) | \
     defined( __AVR_ATtiny45__ ) | \
     defined( __AVR_ATtiny85__ )
#  define DDR_USI             DDRB
#  define PORT_USI            PORTB
#  define PIN_USI             PINB
#  define PORT_USI_SDA        PB0
#  define PORT_USI_SCL        PB2
#  define PIN_USI_SDA         PINB0
#  define PIN_USI_SCL         PINB2
#  define USI_START_COND_INT  USISIF //was USICIF jjg
#  define USI_START_VECTOR    USI_START_vect
#  define USI_OVERFLOW_VECTOR USI_OVF_vect
#endif

#if defined( __AVR_ATtiny26__ )
#  define DDR_USI             DDRB
#  define PORT_USI            PORTB
#  define PIN_USI             PINB
#  define PORT_USI_SDA        PB0
#  define PORT_USI_SCL        PB2
#  define PIN_USI_SDA         PINB0
#  define PIN_USI_SCL         PINB2
#  define USI_START_COND_INT  USISIF
#  define USI_START_VECTOR    USI_STRT_vect
#  define USI_OVERFLOW_VECTOR USI_OVF_vect
#endif

#if defined( __AVR_ATtiny261__ ) | \
     defined( __AVR_ATtiny461__ ) | \
     defined( __AVR_ATtiny861__ )
#  define DDR_USI             DDRB
#  define PORT_USI            PORTB
#  define PIN_USI             PINB
#  define PORT_USI_SDA        PB0
#  define PORT_USI_SCL        PB2
#  define PIN_USI_SDA         PINB0
#  define PIN_USI_SCL         PINB2
#  define USI_START_COND_INT  USISIF
#  define USI_START_VECTOR    USI_START_vect
#  define USI_OVERFLOW_VECTOR USI_OVF_vect
#endif

#if defined( __AVR_ATmega165__ ) | \
     defined( __AVR_ATmega325__ ) | \
     defined( __AVR_ATmega3250__ ) | \
     defined( __AVR_ATmega645__ ) | \
     defined( __AVR_ATmega6450__ ) | \
     defined( __AVR_ATmega329__ ) | \
     defined( __AVR_ATmega3290__ )
#  define DDR_USI             DDRE
#  define PORT_USI            PORTE
#  define PIN_USI             PINE
#  define PORT_USI_SDA        PE5
#  define PORT_USI_SCL        PE4
#  define PIN_USI_SDA         PINE5
#  define PIN_USI_SCL         PINE4
#  define USI_START_COND_INT  USISIF
#  define USI_START_VECTOR    USI_START_vect
#  define USI_OVERFLOW_VECTOR USI_OVERFLOW_vect
#endif

#if defined( __AVR_ATmega169__ )
#  define DDR_USI             DDRE
#  define PORT_USI            PORTE
#  define PIN_USI             PINE
#  define PORT_USI_SDA        PE5
#  define PORT_USI_SCL        PE4
#  define PIN_USI_SDA         PINE5
#  define PIN_USI_SCL         PINE4
#  define USI_START_COND_INT  USISIF
#  define USI_START_VECTOR    USI_START_vect
#  define USI_OVERFLOW_VECTOR USI_OVERFLOW_vect
#endif



/********************************************************************************

                        functions implemented as macros

********************************************************************************/

#define SET_USI_TO_SEND_ACK( ) \
{ \
  /* prepare ACK */ \
  USIDR = 0; \
  /* set SDA as output */ \
  DDR_USI |= ( 1 << PORT_USI_SDA ); \
  /* clear all interrupt flags, except Start Cond */ \
  USISR = \
       ( 0 << USI_START_COND_INT ) | \
       ( 1 << USIOIF ) | ( 1 << USIPF ) | \
       ( 1 << USIDC )| \
       /* set USI counter to shift 1 bit */ \
       ( 0x0E << USICNT0 ); \
}

#define SET_USI_TO_READ_ACK( ) \
{ \
  /* set SDA as input */ \
  DDR_USI &= ~( 1 << PORT_USI_SDA ); \
  /* prepare ACK */ \
  USIDR = 0; \
  /* clear all interrupt flags, except Start Cond */ \
  USISR = \
       ( 0 << USI_START_COND_INT ) | \
       ( 1 << USIOIF ) | \
       ( 1 << USIPF ) | \
       ( 1 << USIDC ) | \
       /* set USI counter to shift 1 bit */ \
       ( 0x0E << USICNT0 ); \
}

#define SET_USI_TO_TWI_START_CONDITION_MODE( ) \
{ \
  USICR = \
       /* enable Start Condition Interrupt, disable Overflow Interrupt */ \
       ( 1 << USISIE ) | ( 0 << USIOIE ) | \
       /* set USI in Two-wire mode, no USI Counter overflow hold */ \
       ( 1 << USIWM1 ) | ( 0 << USIWM0 ) | \
       /* Shift Register Clock Source = External, positive edge */ \
       /* 4-Bit Counter Source = external, both edges */ \
       ( 1 << USICS1 ) | ( 0 << USICS0 ) | ( 0 << USICLK ) | \
       /* no toggle clock-port pin */ \
       ( 0 << USITC ); \
  USISR = \
        /* clear all interrupt flags, except Start Cond */ \
        ( 0 << USI_START_COND_INT ) | ( 1 << USIOIF ) | ( 1 << USIPF ) | \
        ( 1 << USIDC ) | ( 0x0 << USICNT0 ); \
}

#define SET_USI_TO_SEND_DATA456( ) \
{ \
  /* set SDA as output */ \
  DDR_USI |=  ( 1 << PORT_USI_SDA ); \
  /* clear all interrupt flags, except Start Cond */ \
  USISR    =  \
       ( 0 << USI_START_COND_INT ) | ( 1 << USIOIF ) | ( 1 << USIPF ) | \
       ( 1 << USIDC) | \
       /* set USI to shift out 8 bits */ \
       ( 0x0 << USICNT0 ); \
}

#define SET_USI_TO_SEND_DATA( ) \
{ \
  /* set SDA as output */ \
  DDR_USI |=  ( 1 << PORT_USI_SDA ); \  
  /* clear all interrupt flags, except Start Cond */ \
  USISR    =  \
       ( 0 << USI_START_COND_INT ) | ( 0 << USIOIF ) | ( 1 << USIPF ) | \
       ( 1 << USIDC)| \
       /* set USI to shift out 8 bits */ \
       ( 1 << USICNT0 ); \
}

//  | ( 0x0 << USICNT1 )|( 0x0 << USICNT2 )|( 0x0 << USICNT3 ) 
#define SET_USI_TO_SEND_DATA123( ) \
{ \
  /* set SDA as output */ \
  DDR_USI |=  ( 1 << PORT_USI_SDA ); \
  /* clear all interrupt flags, except Start Cond */ \
  USISR    =  (0 << USI_START_COND_INT) | ( 0 << USIOIF ) | ( 1 << USIPF ) | ( 1 << USIDC) | ( 0x0 << USICNT0 )   ; \  
}

#define SET_USI_TO_READ_DATA( ) \
{ \
  /* set SDA as input */ \
  DDR_USI &= ~( 1 << PORT_USI_SDA ); \
  /* clear all interrupt flags, except Start Cond */ \
  USISR    = \
       ( 0 << USI_START_COND_INT ) | ( 1 << USIOIF ) | \
       ( 1 << USIPF ) | ( 1 << USIDC ) | \
       /* set USI to shift out 8 bits */ \
       ( 0x0 << USICNT0 ); \
}



/********************************************************************************

                                   typedef's

********************************************************************************/

typedef enum
{
  USI_SLAVE_CHECK_ADDRESS                = 0x00,
  USI_SLAVE_SEND_DATA                    = 0x01,
  USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA = 0x02,
  USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA   = 0x03,
  USI_SLAVE_REQUEST_DATA                 = 0x04,
  USI_SLAVE_GET_DATA_AND_SEND_ACK        = 0x05
} overflowState_t;



/********************************************************************************

                                local variables

********************************************************************************/
#define T4_TWI 6
static uint8_t                  slaveAddress;
static volatile overflowState_t overflowState;


static volatile uint8_t rxBuf[ TWI_RX_BUFFER_SIZE ];
static volatile uint8_t rxHead;
static volatile uint8_t rxTail;

static volatile uint8_t txBuf[ TWI_TX_BUFFER_SIZE ];
static volatile uint8_t txHead;
static volatile uint8_t txTail;
static volatile uint8_t txCount; // new
static volatile uint8_t rxCount; // new

/********************************************************************************

                                local functions

********************************************************************************/



// flushes the TWI buffers

static
void
flushTwiBuffers(
  void
)
{
  rxTail = 0;
  rxHead = 0;
  txTail = 0;
  txHead = 0;
} // end flushTwiBuffers


void UsiBlink(uint8_t led, uint8_t times){ // poor man's display
  for (uint8_t i=0; i< times; i++){
    digitalWrite(led,1);
    delay (250);
    digitalWrite(led,0);
    delay (250);
  }
}



/********************************************************************************

                                public functions

********************************************************************************/



// initialise USI for TWI slave mode

void
usiTwiSlaveInit(
  uint8_t ownAddress
)
{

  flushTwiBuffers( );
  txBuf[0] = 0x07;
  slaveAddress = ownAddress;

  // In Two Wire mode (USIWM1, USIWM0 = 1X), the slave USI will pull SCL
  // low when a start condition is detected or a counter overflow (only
  // for USIWM1, USIWM0 = 11).  This inserts a wait state.  SCL is released
  // by the ISRs (USI_START_vect and USI_OVERFLOW_vect).

  // Set SCL and SDA as output
  DDR_USI |= ( 1 << PORT_USI_SCL ) | ( 1 << PORT_USI_SDA );

  // set SCL high
  PORT_USI |= ( 1 << PORT_USI_SCL );

  // set SDA high
  PORT_USI |= ( 1 << PORT_USI_SDA );

  // Set SDA as input
  DDR_USI &= ~( 1 << PORT_USI_SDA );

  USICR =
       // enable Start Condition Interrupt
       ( 1 << USISIE ) |
       // disable Overflow Interrupt
       ( 0 << USIOIE ) |
       // set USI in Two-wire mode, no USI Counter overflow hold
       ( 1 << USIWM1 ) | ( 0 << USIWM0 ) |
       // Shift Register Clock Source = external, positive edge
       // 4-Bit Counter Source = external, both edges
       ( 1 << USICS1 ) | ( 0 << USICS0 ) | ( 0 << USICLK ) |
       // no toggle clock-port pin
       ( 0 << USITC );

  // clear all interrupt flags and reset overflow counter

  USISR = ( 1 << USI_START_COND_INT ) | ( 1 << USIOIF ) | ( 1 << USIPF ) | ( 1 << USIDC );

} // end usiTwiSlaveInit










// begin Ringbuffer Helper Classes ******************************
static volatile uint8_t rxBufStart = 0;
static volatile uint8_t rxBufEnd = 0;
static volatile uint8_t rxBufCounter = 0;
volatile bool RxBufferInsertByte(volatile uint8_t data)
{
	if (rxBufCounter < TWI_RX_BUFFER_MASK)
	{		
		rxBuf[rxBufEnd] = data;
		rxBufEnd = (rxBufEnd + 1) & TWI_RX_BUFFER_MASK;
		rxBufCounter++;		
		
		return true;
	}
	else // throw off old stuff to trash
	{
		rxBuf[rxBufEnd] = data;
		rxBufStart = (rxBufStart + 1) & TWI_RX_BUFFER_MASK;
		rxBufEnd = (rxBufEnd + 1) & TWI_RX_BUFFER_MASK;
		
		return false;
	}
	
}

volatile uint8_t RxBufferRemoveByte()
{
	uint8_t retValue = 0xff;
	if (rxBufCounter > 0)
	{
		
		retValue = rxBuf[rxBufStart];
		rxBufStart = (rxBufStart + 1) & TWI_RX_BUFFER_MASK;
		rxBufCounter--;
	}
	return retValue;
}




static volatile uint8_t txBufStart = 0;
static volatile uint8_t txBufEnd = 0;
static volatile uint8_t txBufCounter = 0;
volatile bool TxBufferInsertByte(volatile uint8_t data)
{
	if (txBufCounter < TWI_TX_BUFFER_MASK)
	{		
		txBuf[txBufEnd] = data;
		txBufEnd = (txBufEnd + 1) & TWI_TX_BUFFER_MASK;
		txBufCounter++;		
		
		return true;
	}
	else // throw off old stuff to trash
	{
		txBuf[txBufEnd] = data;
		txBufStart = (txBufStart + 1) & TWI_TX_BUFFER_MASK;
		txBufEnd = (txBufEnd + 1) & TWI_TX_BUFFER_MASK;
		
		return false;
	}
	
}

volatile uint8_t TxBufferRemoveByte()
{
	uint8_t retValue = 0xff;
	if (txBufCounter > 0)
	{
		
		retValue = txBuf[txBufStart];
		txBufStart = (txBufStart + 1) & TWI_TX_BUFFER_MASK;
		txBufCounter--;
	}
	return retValue;
}


// End Ringbuffer Helper Classes ******************************











// put data in the transmission buffer, wait if buffer is full

//static volatile bool txLocked;
//bool IsTxLocked()
//{
//	return txLocked;
//}

//void TxLock()
//{	
	//int x = 0;
	//while(txLocked && x < 60000)
	//{
	//	x++;
	//}
	//if (x > 60000)
	//{
	//	TxUnLock();
	//	txLocked = false;
	//}
	//else{
	//   txLocked = true;
	//}
//}

//void TxUnLock()
//{
	//txLocked = false;
//}

static volatile bool isSending = false;
void usiSetSendModus(bool sendModus)
{
	isSending = sendModus;
}



//static volatile uint8_t txBuffer = 0x00;


void
usiTwiTransmitByte(
  volatile uint8_t data
)
{
	//usiSetSendModus(true);
	//cli();
	//volatile uint8_t tmpData = 0;
  //uint8_t tmphead;
  // calculate buffer index
  //tmphead = ( txHead + 1 ) & TWI_TX_BUFFER_MASK;

  // wait for free space in buffer
  //while(txBufCounter >= TWI_TX_BUFFER_MASK);
 // while ( tmphead == txTail );
  //while ( txBuffer ){}
  //txBuffer = 0x01;
  //TxLock();	
  // store data in buffer
  //txBuf[ tmphead ] = data;
  //ATOMIC_BLOCK(ATOMIC_FORCEON) 
  {  
  //   tmpData = data;  
  }
  //TxLock();
  
  //ATOMIC_BLOCK(ATOMIC_FORCEON)
  {	 
     //txBuf[ 0 ] = tmpData;
	 //txBuf2 = data;
	 // TxBufferInsertByte(0x23);
	 
	 //TxBufferInsertByte(txBufStart);
	 //TxBufferInsertByte(txBufCounter);
	 TxBufferInsertByte(data);
	 
	 
	 
  }
  
  //TxUnLock();
  //sei();
  // store new index
  //txHead = tmphead;  
  //TxUnLock();	
} // end usiTwiTransmitByte



// return a byte from the receive buffer, wait if buffer is empty

uint8_t
usiTwiReceiveByte(void)
{
	//while ( rxHead == rxTail )
  {
		//delay(33);
  }
  // wait for Rx data
  //while (rxBufCounter < 1){}
  
  
  uint8_t returnValue = RxBufferRemoveByte();
  
  
  // return data from the buffer.
  return returnValue;
} // end usiTwiReceiveByte



// check if there is data in the receive buffer

bool
usiTwiDataInReceiveBuffer(
  void
)
{

  // return 0 (false) if the receive buffer is empty
  //return rxHead != rxTail;
  return (rxBufCounter>0);
 //return true;
  
  
  
  //return txBuffer;

} // end usiTwiDataInReceiveBuffer



/********************************************************************************

                            USI Start Condition ISR

********************************************************************************/

ISR( USI_START_VECTOR )
{

  // set default starting conditions for new TWI package
  overflowState = USI_SLAVE_CHECK_ADDRESS;

  // set SDA as input
  DDR_USI &= ~( 1 << PORT_USI_SDA );

  // wait for SCL to go low to ensure the Start Condition has completed (the
  // start detector will hold SCL low ) - if a Stop Condition arises then leave
  // the interrupt to prevent waiting forever - don't use USISR to test for Stop
  // Condition as in Application Note AVR312 because the Stop Condition Flag is
  // going to be set from the last TWI sequence
  while (
       // SCL his high
       ( PIN_USI & ( 1 << PIN_USI_SCL ) ) &&
       // and SDA is low
       !( ( PIN_USI & ( 1 << PIN_USI_SDA ) ) )
  );


  if ( !( PIN_USI & ( 1 << PIN_USI_SDA ) ) )
  {

    // a Stop Condition did not occur

    USICR =
         // keep Start Condition Interrupt enabled to detect RESTART
         ( 1 << USISIE ) |
         // enable Overflow Interrupt
         ( 1 << USIOIE ) |
         // set USI in Two-wire mode, hold SCL low on USI Counter overflow
         ( 1 << USIWM1 ) | ( 1 << USIWM0 ) |
         // Shift Register Clock Source = External, positive edge
         // 4-Bit Counter Source = external, both edges
         ( 1 << USICS1 ) | ( 0 << USICS0 ) | ( 0 << USICLK ) |
         // no toggle clock-port pin
         ( 0 << USITC );

  }
  else
  {

    // a Stop Condition did occur
    USICR =
         // enable Start Condition Interrupt
         ( 1 << USISIE ) |
         // disable Overflow Interrupt
         ( 0 << USIOIE ) |
         // set USI in Two-wire mode, no USI Counter overflow hold
         ( 1 << USIWM1 ) | ( 0 << USIWM0 ) |
         // Shift Register Clock Source = external, positive edge
         // 4-Bit Counter Source = external, both edges
         ( 1 << USICS1 ) | ( 0 << USICS0 ) | ( 0 << USICLK ) |
         // no toggle clock-port pin
         ( 0 << USITC );

  } // end if

  USISR =
       // clear interrupt flags - resetting the Start Condition Flag will
       // release SCL
       ( 1 << USI_START_COND_INT ) | ( 1 << USIOIF ) |
       ( 1 << USIPF ) |( 1 << USIDC ) |
       // set USI to sample 8 bits (count 16 external SCL pin toggles)
       ( 0x0 << USICNT0);

} // end ISR( USI_START_VECTOR )



/********************************************************************************

                                USI Overflow ISR

Handles all the communication.

Only disabled when waiting for a new Start Condition.

********************************************************************************/

ISR( USI_OVERFLOW_VECTOR )
{

  switch ( overflowState )
  {

    // Address mode: check address and send ACK (and next USI_SLAVE_SEND_DATA) if OK,
    // else reset USI
    case USI_SLAVE_CHECK_ADDRESS:
      if ( ( USIDR == 0 ) || ( ( USIDR >> 1 ) == slaveAddress) )
      {
          if ( USIDR & 0x01 )
        {
          overflowState = USI_SLAVE_SEND_DATA;
        }
        else
        {
          overflowState = USI_SLAVE_REQUEST_DATA;
        } // end if
		//_delay_us(T4_TWI);
        SET_USI_TO_SEND_ACK( );
      }
      else
      {
		//_delay_us(T4_TWI*1);
        SET_USI_TO_TWI_START_CONDITION_MODE( );
      }
      break;

    // Master write data mode: check reply and goto USI_SLAVE_SEND_DATA if OK,
    // else reset USI
    case USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA:
	  
      if ( USIDR )
      {
        // if NACK, the master does not want more data
		//_delay_us(T4_TWI*1);
        SET_USI_TO_TWI_START_CONDITION_MODE( );		
        return;
      }
	  
      // from here we just drop straight into USI_SLAVE_SEND_DATA if the
      // master sent an ACK













    // copy data from buffer to USIDR and set USI to shift byte
    // next USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA
    case USI_SLAVE_SEND_DATA:
      // Get data from Buffer      
	  if (txBufCounter > 0 )		  
      {
		uint8_t tmpData;
		tmpData = TxBufferRemoveByte();
	   
		//_delay_us(T4_TWI*1);
		//if(0)	
        
	    //USISR |= (0<<USIOIF); //Überlaufflag löschen 
	    //// clear Interrupt Flags and Counter.
        //USISR = (1 << USISIF) | (1 << USIOIF) | (1 << USIPF) | (1 << USIDC);
    
	DDR_USI |=  ( 0 << PORT_USI_SDA ); \  
  /* clear all interrupt flags, except Start Cond */ \
  USISR    =  ( 0 << USI_START_COND_INT ) | ( 0 << USIOIF ) | ( 0 << USIPF ) | ( 0 << USIDC)| ( 1 << USICNT0 );	
	
	    
        //DDR_USI |=  ( 1 << PORT_USI_SDA ); /* set SDA as output */  
	//	USIDR = 0x00;
		//USIDR = 0xFF;
		USIDR = 0x002;
	//	USIDR = tmpData; //tmpData;
		//USISR |= (0<<USIOIF); //Überlaufflag löschen 
				
		
		//_delay_us(88);
		//while(USIOIF==0);
		
		overflowState = USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA;	SET_USI_TO_SEND_DATA( );
	  }
      else
      {
        // the buffer is empty
        _delay_us(T4_TWI*3);
		SET_USI_TO_TWI_START_CONDITION_MODE( );
		// //txBuffer = 0x00;
        return;
      } // end if
      
      break;
    // set USI to sample reply from master
    // next USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA
    case USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA:
      overflowState = USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA;
	  //_delay_us(T4_TWI);
      SET_USI_TO_READ_ACK( );
      break;

    // Master read data mode: set USI to sample data from master, next
    // USI_SLAVE_GET_DATA_AND_SEND_ACK
    case USI_SLAVE_REQUEST_DATA:	  
      overflowState = USI_SLAVE_GET_DATA_AND_SEND_ACK;
	  //_delay_us(T4_TWI);
      SET_USI_TO_READ_DATA( );	  
      break;

    // copy data from USIDR and send ACK
    // next USI_SLAVE_REQUEST_DATA
    case USI_SLAVE_GET_DATA_AND_SEND_ACK:
	
  // put data into buffer
    // Not necessary, but prevents warnings
	if(true)
	{
	// put data into buffer
        uint8_t tmpRxBuf = 0;
	    //ATOMIC_BLOCK(ATOMIC_FORCEON)
		//{
		   tmpRxBuf = USIDR;
		//}
	    RxBufferInsertByte(tmpRxBuf);
	}	 
	
      //usiSetSendModus(false);
	  // next USI_SLAVE_REQUEST_DATA
      overflowState = USI_SLAVE_REQUEST_DATA;
	  //_delay_us(T4_TWI);
      SET_USI_TO_SEND_ACK( );	  
      break;

  } // end switch

} // end ISR( USI_OVERFLOW_VECTOR )
